@@ -24,6 +24,7 @@
 #include "sat-solver/sat-encoder.h"
 #include "corpus/corpus.h"
 #include "spellcheck.h"
+#include "utilities.h"
 
 /***************************************************************
 *
@@ -116,7 +117,7 @@ Parse_Options parse_options_create(void)
 	po->screen_width = 79;
 	po->display_on = TRUE;
 	po->display_postscript = FALSE;
-	po->display_constituents = 0;
+	po->display_constituents = NO_DISPLAY;
 	po->display_bad = FALSE;
 	po->display_disjuncts = FALSE;
 	po->display_links = FALSE;
@@ -176,10 +177,19 @@ void parse_options_set_use_sat_parser(Parse_Options opts, int dummy) {
 	          " library was built without SAT solver support.\n");
 #endif
 }
+
 int parse_options_get_use_sat_parser(Parse_Options opts) {
 	return opts->use_sat_solver;
 }
 
+void parse_options_set_use_viterbi(Parse_Options opts, int dummy) {
+	opts->use_viterbi = dummy;
+}
+
+int parse_options_get_use_viterbi(Parse_Options opts) {
+	return opts->use_viterbi;
+}
+
 void parse_options_set_use_fat_links(Parse_Options opts, int dummy) {
 	opts->use_fat_links = dummy;
 }
@@ -351,20 +361,21 @@ int parse_options_get_display_postscript(Parse_Options opts)
 	return opts->display_postscript;
 }
 
-void parse_options_set_display_constituents(Parse_Options opts, int dummy)
+void parse_options_set_display_constituents(Parse_Options opts, ConstituentDisplayStyle dummy)
 {
-	if ((dummy < 0) || (dummy > 3)) {
+	if (dummy > MAX_STYLES)
+	{
 		prt_error("Possible values for constituents: \n"
 	             "   0 (no display)\n"
 	             "   1 (treebank style, multi-line indented)\n"
 	             "   2 (flat tree, square brackets)\n"
 	             "   3 (flat treebank style)\n");
-	   opts->display_constituents = 0;
+	   opts->display_constituents = NO_DISPLAY;
 	}
 	else opts->display_constituents = dummy;
 }
 
-int parse_options_get_display_constituents(Parse_Options opts)
+ConstituentDisplayStyle parse_options_get_display_constituents(Parse_Options opts)
 {
 	return opts->display_constituents;
 }
@@ -433,353 +444,6 @@ void parse_options_reset_resources(Parse_Options opts) {
 	resources_reset(opts->resources);
 }
 
-
-/***************************************************************
-*
-* Routines for manipulating Dictionary
-*
-****************************************************************/
-
-/* Units will typically have a ".u" at the end. Get
- * rid of it, as otherwise stipping is messed up. */
-static inline char * deinflect(const char * str)
-{
-	char * s = strdup(str);
-	char * p = strchr(s, '.');
-	if (p && p != s) *p = 0x0;
-	return s;
-}
-
-static void affix_list_create(Dictionary dict)
-{
-	int i, j, k, l, m;
-	int r_strippable=0, l_strippable=0, u_strippable=0;
-	int s_strippable=0, p_strippable=0;
-	Dict_node * dn, * dn2, * start_dn;
-
-	const char * rpunc_con = "RPUNC";
-	const char * lpunc_con = "LPUNC";
-	const char * units_con = "UNITS";
-
-	/* Hmm SUF and PRE do not seem to be used at this time ... */
-	const char * suf_con = "SUF";
-	const char * pre_con = "PRE";
-
-	dict->strip_left = NULL;
-	dict->strip_right = NULL;
-	dict->strip_units = NULL;
-	dict->prefix = NULL;
-	dict->suffix = NULL;
-
-	/* Load affixes from the affix table. 
-	 */
-	start_dn = list_whole_dictionary(dict->root, NULL);
-	for (dn = start_dn; dn != NULL; dn = dn->right)
-	{
-		if (word_has_connector(dn, rpunc_con, 0)) r_strippable++;
-		if (word_has_connector(dn, lpunc_con, 0)) l_strippable++;
-		if (word_has_connector(dn, units_con, 0)) u_strippable++;
-		if (word_has_connector(dn, suf_con, 0)) s_strippable++;
-		if (word_has_connector(dn, pre_con, 0)) p_strippable++;
-	}
-	dict->strip_right = (const char **) xalloc(r_strippable * sizeof(char *));
-	dict->strip_left = (const char **) xalloc(l_strippable * sizeof(char *));
-	dict->strip_units = (const char **) xalloc(u_strippable * sizeof(char *));
-	dict->suffix = (const char **) xalloc(s_strippable * sizeof(char *));
-	dict->prefix = (const char **) xalloc(p_strippable * sizeof(char *));
-
-	dict->r_strippable = r_strippable;
-	dict->l_strippable = l_strippable;
-	dict->u_strippable = u_strippable;
-	dict->p_strippable = p_strippable;
-	dict->s_strippable = s_strippable;
-
-	i = 0;
-	j = 0;
-	k = 0;
-	l = 0;
-	m = 0;
-	dn = start_dn;
-
-	while (dn != NULL)
-	{
-		if (word_has_connector(dn, rpunc_con, 0))
-		{
-			dict->strip_right[i] = deinflect(dn->string);
-			i++;
-		}
-		if (word_has_connector(dn, lpunc_con, 0))
-		{
-			dict->strip_left[j] = deinflect(dn->string);
-			j++;
-		}
-		if (word_has_connector(dn, units_con, 0))
-		{
-			dict->strip_units[m] = deinflect(dn->string);
-			m++;
-		}
-		if (word_has_connector(dn, suf_con, 0))
-		{
-			dict->suffix[k] = dn->string;
-			k++;
-		}
-		if (word_has_connector(dn, pre_con, 0))
-		{
-			dict->prefix[l] = dn->string;
-			l++;
-		}
-		dn2 = dn->right;
-		dn->right = NULL;
-		xfree(dn, sizeof(Dict_node));
-		dn = dn2;
-	}
-}
-
-static void affix_list_delete(Dictionary dict)
-{
-	int i;
-	for (i=0; i<dict->l_strippable; i++)
-	{
-		free((char *)dict->strip_left[i]);
-	}
-	for (i=0; i<dict->r_strippable; i++)
-	{
-		free((char *)dict->strip_right[i]);
-	}
-	for (i=0; i<dict->u_strippable; i++)
-	{
-		free((char *)dict->strip_units[i]);
-	}
-	xfree(dict->strip_right, dict->r_strippable * sizeof(char *));
-	xfree(dict->strip_left, dict->l_strippable * sizeof(char *));
-	xfree(dict->strip_units, dict->u_strippable * sizeof(char *));
-	xfree(dict->suffix, dict->s_strippable * sizeof(char *));
-	xfree(dict->prefix, dict->p_strippable * sizeof(char *));
-}
-
-/**
- * The following function is dictionary_create with an extra
- * paramater called "path". If this is non-null, then the path
- * used to find the file is taken from that path. Otherwise,
- * the path is taken from the dict_name.  This is only needed
- * because an affix_file is opened by a recursive call to this
- * function.
- */
-static Dictionary
-dictionary_six(const char * lang, const char * dict_name,
-                const char * pp_name, const char * cons_name,
-                const char * affix_name, const char * regex_name)
-{
-	const char * t;
-	Dictionary dict;
-	Dict_node *dict_node;
-
-	init_memusage();
-
-	dict = (Dictionary) xalloc(sizeof(struct Dictionary_s));
-	memset(dict, 0, sizeof(struct Dictionary_s));
-
-	dict->string_set = string_set_create();
-	
-	dict->lang = lang;
-	t = strrchr (lang, '/');
-	if (t) dict->lang = string_set_add(t+1, dict->string_set);
-	dict->name = string_set_add(dict_name, dict->string_set);
-
-	dict->max_cost = 1000;
-	dict->num_entries = 0;
-	dict->is_special = FALSE;
-	dict->already_got_it = '\0';
-	dict->line_number = 1;
-	dict->root = NULL;
-	dict->word_file_header = NULL;
-	dict->exp_list = NULL;
-	dict->affix_table = NULL;
-	dict->recursive_error = FALSE;
-
-	/* To disable spell-checking, just set the cheker to NULL */
-	dict->spell_checker = spellcheck_create(dict->lang);
-
-	dict->fp = dictopen(dict->name, "r");
-	if (dict->fp == NULL)
-	{
-		prt_error("Error: Could not open dictionary %s\n", dict_name);
-		goto failure;
-	}
-
-	if (!read_dictionary(dict))
-	{
-		fclose(dict->fp);
-		goto failure;
-	}
-	fclose(dict->fp);
-
-	dict->affix_table = NULL;
-	if (affix_name != NULL)
-	{
-		dict->affix_table = dictionary_six(lang, affix_name, NULL, NULL, NULL, NULL);
-		if (dict->affix_table == NULL)
-		{
-			goto failure;
-		}
-		affix_list_create(dict->affix_table);
-	}
-
-	dict->regex_root = NULL;
-	if (regex_name != NULL)
-	{
-		int rc;
-		rc = read_regex_file(dict, regex_name);
-		if (rc) goto failure;
-		rc = compile_regexs(dict);
-		if (rc) goto failure;
-	}
-
-#if USE_CORPUS
-	dict->corpus = NULL;
-	if (affix_name != NULL) /* Don't do this for the second time */
-	{
-		dict->corpus = lg_corpus_new();
-	}
-#endif
-
-	dict->left_wall_defined  = boolean_dictionary_lookup(dict, LEFT_WALL_WORD);
-	dict->right_wall_defined = boolean_dictionary_lookup(dict, RIGHT_WALL_WORD);
-	dict->postprocessor	  = post_process_open(pp_name);
-	dict->constituent_pp	 = post_process_open(cons_name);
-
-	dict->unknown_word_defined = boolean_dictionary_lookup(dict, UNKNOWN_WORD);
-	dict->use_unknown_word = TRUE;
-
-#if DONT_USE_REGEX_GUESSING
-	dict->capitalized_word_defined = boolean_dictionary_lookup(dict, PROPER_WORD);
-	dict->pl_capitalized_word_defined = boolean_dictionary_lookup(dict, PL_PROPER_WORD);
-
-	dict->hyphenated_word_defined = boolean_dictionary_lookup(dict, HYPHENATED_WORD);
-	dict->number_word_defined = boolean_dictionary_lookup(dict, NUMBER_WORD);
-
-	dict->ing_word_defined = boolean_dictionary_lookup(dict, ING_WORD);
-	dict->s_word_defined = boolean_dictionary_lookup(dict, S_WORD);
-	dict->ed_word_defined = boolean_dictionary_lookup(dict, ED_WORD);
-	dict->ly_word_defined = boolean_dictionary_lookup(dict, LY_WORD);
-#endif /* DONT_USE_REGEX_GUESSING */
-
-	if ((dict_node = dictionary_lookup_list(dict, ANDABLE_CONNECTORS_WORD)) != NULL) {
-		dict->andable_connector_set = connector_set_create(dict_node->exp);
-	} else {
-		dict->andable_connector_set = NULL;
-	}
-	free_lookup_list(dict_node);
-
-	if ((dict_node = dictionary_lookup_list(dict, UNLIMITED_CONNECTORS_WORD)) != NULL) {
-		dict->unlimited_connector_set = connector_set_create(dict_node->exp);
-	} else {
-		dict->unlimited_connector_set = NULL;
-	}
-	free_lookup_list(dict_node);
-
-	return dict;
-
-failure:
-	string_set_delete(dict->string_set);
-	xfree(dict, sizeof(struct Dictionary_s));
-	return NULL;
-}
-
-Dictionary
-dictionary_create(const char * dict_name, const char * pp_name,
-                  const char * cons_name, const char * affix_name)
-{
-	return dictionary_six("en", dict_name, pp_name, cons_name, affix_name, NULL);
-}
-
-Dictionary dictionary_create_lang(const char * lang)
-{
-	Dictionary dictionary;
-
-	if(lang && *lang)
-	{
-		char * dict_name;
-		char * pp_name;
-		char * cons_name;
-		char * affix_name;
-		char * regex_name;
-	
-		dict_name = join_path(lang, "4.0.dict");
-		pp_name = join_path(lang, "4.0.knowledge");
-		cons_name = join_path(lang, "4.0.constituent-knowledge");
-		affix_name = join_path(lang, "4.0.affix");
-		regex_name = join_path(lang, "4.0.regex");
-	
-		dictionary = dictionary_six(lang, dict_name, pp_name, cons_name,
-		                             affix_name, regex_name);
-	
-		free(regex_name);
-		free(affix_name);
-		free(cons_name);
-		free(pp_name);
-		free(dict_name);
-	}
-	else
-	{
-		prt_error("Error: No language specified!\n");
-		dictionary = NULL;
-	}
-
-	return dictionary;
-}
-
-Dictionary dictionary_create_default_lang(void)
-{
-	Dictionary dictionary;
-	char * lang;
-
-	lang = get_default_locale();
-	if(lang && *lang) {
-		dictionary = dictionary_create_lang(lang);
-		free(lang);
-	} else {
-		/* Default to en when locales are broken (e.g. WIN32) */
-		dictionary = dictionary_create_lang("en");
-	}
-
-	return dictionary;
-}
-
-int dictionary_delete(Dictionary dict)
-{
-	if (verbosity > 0) {
-		prt_error("Info: Freeing dictionary %s\n", dict->name);
-	}
-
-#if USE_CORPUS
-	lg_corpus_delete(dict->corpus);
-#endif
-
-	if (dict->affix_table != NULL) {
-		affix_list_delete(dict->affix_table);
-		dictionary_delete(dict->affix_table);
-	}
-	spellcheck_destroy(dict->spell_checker);
-
-	connector_set_delete(dict->andable_connector_set);
-	connector_set_delete(dict->unlimited_connector_set);
-
-	post_process_close(dict->postprocessor);
-	post_process_close(dict->constituent_pp);
-	string_set_delete(dict->string_set);
-	free_regexs(dict);
-	free_dictionary(dict);
-	xfree(dict, sizeof(struct Dictionary_s));
-
-	return 0;
-}
-
-int dictionary_get_max_cost(Dictionary dict)
-{
-	return dict->max_cost;
-}
-
 /***************************************************************
 *
 * Routines for postprocessing
@@ -850,9 +514,12 @@ static void post_process_linkages(Sentence sent, Parse_Options opts)
 	int N_linkages_found, N_linkages_alloced;
 	int N_linkages_post_processed, N_valid_linkages;
 	int N_thin_linkages;
-	int overflowed, only_canonical_allowed;
+	int overflowed;
 	Linkage_info *link_info;
+#ifdef USE_FAT_LINKAGES
+	int only_canonical_allowed;
 	int canonical;
+#endif /* USE_FAT_LINKAGES */
 
 	free_post_processing(sent);
 
@@ -921,7 +588,9 @@ static void post_process_linkages(Sentence sent, Parse_Options opts)
 		}
 	}
 
+#ifdef USE_FAT_LINKAGES
 	only_canonical_allowed = !(overflowed || (N_linkages_found > 2*opts->linkage_limit));
+#endif /* USE_FAT_LINKAGES */
 	/* When we're processing only a small subset of the linkages,
 	 * don't worry about restricting the set we consider to be
 	 * canonical ones.  In the extreme case where we are only
@@ -941,12 +610,14 @@ static void post_process_linkages(Sentence sent, Parse_Options opts)
 				   (!resources_exhausted(opts->resources)); in++)
 		{
 			extract_links(indices[in], sent->null_count, sent->parse_info);
+#ifdef USE_FAT_LINKAGES
 			if (set_has_fat_down(sent))
 			{
 				if (only_canonical_allowed && !is_canonical_linkage(sent)) continue;
 				analyze_fat_linkage(sent, opts, PP_FIRST_PASS);
 			}
 			else
+#endif /* USE_FAT_LINKAGES */
 			{
 				analyze_thin_linkage(sent, opts, PP_FIRST_PASS);
 			}
@@ -961,6 +632,7 @@ static void post_process_linkages(Sentence sent, Parse_Options opts)
 	{
 		Linkage_info *lifo = &link_info[N_linkages_post_processed];
 		extract_links(indices[in], sent->null_count, sent->parse_info);
+#ifdef USE_FAT_LINKAGES
 		if (set_has_fat_down(sent))
 		{
 			canonical = is_canonical_linkage(sent);
@@ -970,6 +642,7 @@ static void post_process_linkages(Sentence sent, Parse_Options opts)
 			lifo->canonical = canonical;
 		}
 		else
+#endif /* USE_FAT_LINKAGES */
 		{
 			*lifo = analyze_thin_linkage(sent, opts, PP_SECOND_PASS);
 			lifo->fat = FALSE;
@@ -1052,18 +725,18 @@ Sentence sentence_create(const char *input_string, Dictionary dict)
 	sent->num_linkages_post_processed = 0;
 	sent->num_valid_linkages = 0;
 	sent->link_info = NULL;
-	sent->deletable = NULL;
-	sent->effective_dist = NULL;
 	sent->num_valid_linkages = 0;
 	sent->null_count = 0;
 	sent->parse_info = NULL;
 	sent->string_set = string_set_create();
 
 	sent->q_pruned_rules = FALSE;
-	sent->is_conjunction = NULL;
-
-	sent->dptr = NULL;
+#ifdef USE_FAT_LINKAGES
 	sent->deletable = NULL;
+	sent->is_conjunction = NULL;
+	sent->dptr = NULL;
+	sent->effective_dist = NULL;
+#endif /* USE_FAT_LINKAGES */
 
 	/* Make a copy of the input */
 	sent->orig_sentence = string_set_add (input_string, sent->string_set);
@@ -1071,6 +744,7 @@ Sentence sentence_create(const char *input_string, Dictionary dict)
 	return sent;
 }
 
+#ifdef USE_FAT_LINKAGES
 /* XXX Extreme hack alert -- English-language words are used
  * completely naked in the C source code!!! FIXME !!!!
  */
@@ -1087,6 +761,7 @@ static void set_is_conjunction(Sentence sent)
 			(strcmp(s, "nor")==0);
 	}
 }
+#endif /* USE_FAT_LINKAGES */
 
 int sentence_split(Sentence sent, Parse_Options opts)
 {
@@ -1097,7 +772,9 @@ int sentence_split(Sentence sent, Parse_Options opts)
 	 * routines depend on sent-length, which might change in different
 	 * parse-opts settings. 
 	 */
+#ifdef USE_FAT_LINKAGES
 	free_deletable(sent);
+#endif /* USE_FAT_LINKAGES */
 
 	/* Tokenize */
 	if (!separate_sentence(sent, opts))
@@ -1106,9 +783,11 @@ int sentence_split(Sentence sent, Parse_Options opts)
 	}
 
 	sent->q_pruned_rules = FALSE; /* for post processing */
+#ifdef USE_FAT_LINKAGES
 	sent->is_conjunction = (char *) xalloc(sizeof(char)*sent->length);
 	set_is_conjunction(sent);
 	initialize_conjunction_tables(sent);
+#endif /* USE_FAT_LINKAGES */
 
 	for (i=0; i<sent->length; i++)
 	{
@@ -1140,18 +819,24 @@ void sentence_delete(Sentence sent)
 {
 	if (!sent) return;
 	sat_sentence_delete(sent);
-	/* free_andlists(sent); */
+#ifdef USE_FAT_LINKAGES
 	free_sentence_disjuncts(sent);
+#endif /* USE_FAT_LINKAGES */
 	free_sentence_expressions(sent);
 	string_set_delete(sent->string_set);
 	if (sent->parse_info) free_parse_info(sent->parse_info);
 	free_post_processing(sent);
 	post_process_close_sentence(sent->dict->postprocessor);
+#ifdef USE_FAT_LINKAGES
 	free_deletable(sent);
 	free_effective_dist(sent);
+#endif /* USE_FAT_LINKAGES */
+
 	free_count(sent);
 	free_analyze(sent);
+#ifdef USE_FAT_LINKAGES
 	if (sent->is_conjunction) xfree(sent->is_conjunction, sizeof(char)*sent->length);
+#endif /* USE_FAT_LINKAGES */
 	xfree((char *) sent, sizeof(struct Sentence_s));
 }
 
@@ -1318,7 +1003,9 @@ int sentence_parse(Sentence sent, Parse_Options opts)
 	}
 
 	/* Initialize/free any leftover garbage */
+#ifdef USE_FAT_LINKAGES
 	free_sentence_disjuncts(sent);
+#endif /* USE_FAT_LINKAGES */
 	resources_reset_space(opts->resources);
 
 	if (resources_exhausted(opts->resources)) {
@@ -1378,11 +1065,13 @@ Linkage linkage_create(int k, Sentence sent, Parse_Options opts)
 	extract_links(sent->link_info[k].index, sent->null_count, sent->parse_info);
 	compute_chosen_words(sent, linkage);
 
+#ifdef USE_FAT_LINKAGES
 	if (set_has_fat_down(sent))
 	{
 		extract_fat_linkage(sent, opts, linkage);
 	}
 	else
+#endif /* USE_FAT_LINKAGES */
 	{
 		extract_thin_linkage(sent, opts, linkage);
 	}
@@ -1458,6 +1147,7 @@ void linkage_delete(Linkage linkage)
 	exfree(linkage, sizeof(struct Linkage_s));
 }
 
+#ifdef USE_FAT_LINKAGES
 static int links_are_equal(Link *l, Link *m)
 {
 	return ((l->l == m->l) && (l->r == m->r) && (strcmp(l->name, m->name)==0));
@@ -1489,7 +1179,6 @@ static PP_info excopy_pp_info(PP_info ppi)
 	return newppi;
 }
 
-
 static Sublinkage unionize_linkage(Linkage linkage)
 {
 	int i, j, num_in_union=0;
@@ -1533,9 +1222,11 @@ static Sublinkage unionize_linkage(Linkage linkage)
 
 	return u;
 }
+#endif /* USE_FAT_LINKAGES */
 
 int linkage_compute_union(Linkage linkage)
 {
+#ifdef USE_FAT_LINKAGES
 	int i, num_subs=linkage->num_sublinkages;
 	Sublinkage * new_sublinkage, *s;
 
@@ -1569,6 +1260,9 @@ int linkage_compute_union(Linkage linkage)
 	linkage->unionized = TRUE;
 	linkage->current = linkage->num_sublinkages-1;
 	return 1;
+#else
+	return 0;
+#endif /* USE_FAT_LINKAGES */
 }
 
 int linkage_get_num_sublinkages(const Linkage linkage)
@@ -1730,16 +1424,24 @@ int linkage_disjunct_cost(const Linkage linkage)
 
 int linkage_is_fat(const Linkage linkage)
 {
+#ifdef USE_FAT_LINKAGES
 	/* The sat solver (currently) fails to fill in info */
 	if (!linkage->info) return 0;
 	return linkage->info->fat;
+#else
+	return FALSE;
+#endif /* USE_FAT_LINKAGES */
 }
 
 int linkage_and_cost(Linkage linkage)
 {
+#ifdef USE_FAT_LINKAGES
 	/* The sat solver (currently) fails to fill in info */
 	if (!linkage->info) return 0;
 	return linkage->info->and_cost;
+#else
+	return 0;
+#endif /* USE_FAT_LINKAGES */
 }
 
 int linkage_link_cost(const Linkage linkage)
@@ -1786,9 +1488,13 @@ int linkage_is_canonical(const Linkage linkage)
 
 int linkage_is_improper(const Linkage linkage)
 {
+#ifdef USE_FAT_LINKAGES
 	/* The sat solver (currently) fails to fill in info */
 	if (!linkage->info) return FALSE;
 	return linkage->info->improper_fat_linkage;
+#else
+	return FALSE;
+#endif /* USE_FAT_LINKAGES */
 }
 
 int linkage_has_inconsistent_domains(const Linkage linkage)
@@ -1832,11 +1538,13 @@ void linkage_post_process(Linkage linkage, Postprocessor * postprocessor)
 			subl->violation = NULL;
 		}
 
+#ifdef USE_FAT_LINKAGES
 		if (linkage->info->improper_fat_linkage)
 		{
 			pp = NULL;
 		}
 		else
+#endif /* USE_FAT_LINKAGES */
 		{
 			pp = post_process(postprocessor, opts, sent, subl, FALSE);
 			/* This can return NULL, for example if there is no
