@@ -15,9 +15,11 @@ s64  table_lookup(Sentence, int, int, Connector *, Connector *, int);
 int  do_match(Sentence, Connector *a, Connector *b, int wa, int wb);
 int  x_match(Sentence, Connector *a, Connector *b);
 s64  do_parse(Sentence sent, int mincost, Parse_Options opts);
+#ifdef USE_FAT_LINKAGES
 void conjunction_prune(Sentence sent, Parse_Options opts);
 void count_set_effective_distance(Sentence sent);
 void count_unset_effective_distance(Sentence sent);
+#endif /* USE_FAT_LINKAGES */
 void delete_unmarked_disjuncts(Sentence sent);
 
 void init_count(Sentence sent);
